@@ -42,7 +42,7 @@ def find_day(arr, t=None, which='first'):
         inds = [all_inds[0]]
     elif which == 'last':
         inds = [all_inds[-1]]
-    else:
+    else: # pragma: no cover
         errormsg = f'Argument "which" must be "first", "last", or "all", not "{which}"'
         raise ValueError(errormsg)
     return inds
@@ -88,7 +88,7 @@ class Intervention:
         label (str): a label for the intervention (used for plotting, and for ease of identification)
         show_label (bool): whether or not to include the label, if provided, in the legend
         do_plot (bool): whether or not to plot the intervention
-        line_args (dict): arguments passed to pl.axvline() whe plotting
+        line_args (dict): arguments passed to pl.axvline() when plotting
     '''
     def __init__(self, label=None, show_label=True, do_plot=None, line_args=None):
         self.label = label # e.g. "Close schools"
@@ -145,7 +145,7 @@ class Intervention:
 
     def apply(self, sim):
         '''
-        Apply the intervention. This is the core method which each drived intervention
+        Apply the intervention. This is the core method which each derived intervention
         class must implement. This method gets called at each timestep and can make
         arbitrary changes to the Sim object, as well as storing or modifying the
         state of the intervention.
@@ -250,14 +250,14 @@ class dynamic_pars(Intervention):
         subkeys = ['days', 'vals']
         for parkey in pars.keys():
             for subkey in subkeys:
-                if subkey not in pars[parkey].keys():
+                if subkey not in pars[parkey].keys(): # pragma: no cover
                     errormsg = f'Parameter {parkey} is missing subkey {subkey}'
                     raise sc.KeyNotFoundError(errormsg)
                 if sc.isnumber(pars[parkey][subkey]): # Allow scalar values or dicts, but leave everything else unchanged
                     pars[parkey][subkey] = sc.promotetoarray(pars[parkey][subkey])
             len_days = len(pars[parkey]['days'])
             len_vals = len(pars[parkey]['vals'])
-            if len_days != len_vals:
+            if len_days != len_vals: # pragma: no cover
                 raise ValueError(f'Length of days ({len_days}) does not match length of values ({len_vals}) for parameter {parkey}')
         self.pars = pars
         return
@@ -350,7 +350,7 @@ def process_changes(sim, changes, days):
     Ensure lists of changes are in consistent format. Used by change_beta and clip_edges.
     '''
     changes = sc.promotetoarray(changes)
-    if len(days) != len(changes):
+    if len(days) != len(changes): # pragma: no cover
         errormsg = f'Number of days supplied ({len(days)}) does not match number of changes ({len(changes)})'
         raise ValueError(errormsg)
     return changes
@@ -473,7 +473,7 @@ class clip_edges(Intervention):
             # Do the contact moving
             for lkey in self.layers:
                 s_layer = sim.people.contacts[lkey] # Contact layer in the sim
-                i_layer = self.contacts[lkey] # Contat layer in the intervention
+                i_layer = self.contacts[lkey] # Contact layer in the intervention
                 n_sim = len(s_layer) # Number of contacts in the simulation layer
                 n_int = len(i_layer) # Number of contacts in the intervention layer
                 n_contacts = n_sim + n_int # Total number of contacts
@@ -491,7 +491,7 @@ class clip_edges(Intervention):
                         inds = cvu.choose(max_n=n_int, n=abs(n_to_move))
                         to_move = i_layer.pop_inds(inds)
                         s_layer.append(to_move)
-                else:
+                else: # pragma: no cover
                     print(f'Warning: clip_edges() was applied to layer "{lkey}", but no edges were found; please check sim.people.contacts["{lkey}"]')
 
         # Ensure the edges get deleted at the end
@@ -527,7 +527,7 @@ def process_daily_data(daily_data, sim, start_day, as_int=False):
         if daily_data == 'data':
             daily_data = sim.data['new_tests'] # Use default name
         else:
-            try:
+            try: # pragma: no cover
                 daily_data = sim.data[daily_data]
             except Exception as E:
                 errormsg = f'Tried to load testing data from sim.data["{daily_data}"], but that failed: {str(E)}.\nPlease ensure data are loaded into the sim and the column exists.'
@@ -563,7 +563,7 @@ def get_subtargets(subtarget, sim):
     if callable(subtarget):
         subtarget = subtarget(sim)
 
-    if 'inds' not in subtarget:
+    if 'inds' not in subtarget: # pragma: no cover
         errormsg = f'The subtarget dict must have keys "inds" and "vals", but you supplied {subtarget}'
         raise ValueError(errormsg)
 
@@ -579,7 +579,7 @@ def get_subtargets(subtarget, sim):
     else:
         subtarget_vals = subtarget['vals'] # The indices are supplied directly
     if sc.isiterable(subtarget_vals):
-        if len(subtarget_vals) != len(subtarget_inds):
+        if len(subtarget_vals) != len(subtarget_inds): # pragma: no cover
             errormsg = f'Length of subtargeting indices ({len(subtarget_inds)}) does not match length of values ({len(subtarget_vals)})'
             raise ValueError(errormsg)
 
@@ -611,7 +611,7 @@ def get_quar_inds(quar_policy, sim):
         quar_test_inds = np.unique(np.concatenate([cvu.true(sim.people.date_quarantined==t-1-q) for q in quar_policy]))
     elif callable(quar_policy):
         quar_test_inds = quar_policy(sim)
-    else:
+    else: # pragma: no cover
         errormsg = f'Quarantine policy "{quar_policy}" not recognized: must be a string (start, end, both, daily), int, list, array, set, tuple, or function'
         raise ValueError(errormsg)
     return quar_test_inds
@@ -849,8 +849,6 @@ class test_prob(Intervention):
         symp_quar_inds  = np.intersect1d(quar_test_inds, symp_inds)
         asymp_quar_inds = np.intersect1d(quar_test_inds, asymp_inds)
         diag_inds       = cvu.true(sim.people.diagnosed)
-        if self.subtarget is not None:
-            subtarget_inds  = self.subtarget['inds']
 
         # Construct the testing probabilities piece by piece -- complicated, since need to do it in the right order
         test_probs = np.zeros(sim.n) # Begin by assigning equal testing probability to everyone
@@ -1034,7 +1032,7 @@ class vaccine(Intervention):
     '''
     Apply a vaccine to a subset of the population. In addition to changing the
     relative susceptibility and the probability of developing symptoms if still
-    infected, this sintervention stores several types of data:
+    infected, this intervention stores several types of data:
 
         - ``vaccinations``:      the number of vaccine doses per person
         - ``vaccination_dates``: list of dates per person
