@@ -58,13 +58,13 @@ def make_pars(set_prognoses=False, prog_by_age=True, version=None, **kwargs):
 
     # Efficacy of protection measures
     pars['asymp_factor'] = 1.0 # Multiply beta by this factor for asymptomatic cases; no statistically significant difference in transmissibility: https://www.sciencedirect.com/science/article/pii/S1201971220302502
-    pars['iso_factor']   = None # Multiply beta by this factor for diganosed cases to represent isolation; set by reset_layer_pars() below
+    pars['iso_factor']   = None # Multiply beta by this factor for diagnosed cases to represent isolation; set by reset_layer_pars() below
     pars['quar_factor']  = None # Quarantine multiplier on transmissibility and susceptibility; set by reset_layer_pars() below
     pars['quar_period']  = 14  # Number of days to quarantine for; assumption based on standard policies
 
     # Duration parameters: time for disease progression
     pars['dur'] = {}
-    pars['dur']['exp2inf']  = dict(dist='lognormal_int', par1=4.6, par2=4.8) # Duration from exposed to infectious; see Lauer et al., https://www.ncbi.nlm.nih.gov/pmc/articles/PMC7081172/, subtracting inf2sim duration
+    pars['dur']['exp2inf']  = dict(dist='lognormal_int', par1=4.6, par2=4.8) # Duration from exposed to infectious; see Lauer et al., https://www.ncbi.nlm.nih.gov/pmc/articles/PMC7081172/, subtracting inf2sym duration
     pars['dur']['inf2sym']  = dict(dist='lognormal_int', par1=1.0, par2=0.9) # Duration from infectious to symptomatic; see Linton et al., https://doi.org/10.3390/jcm9020538
     pars['dur']['sym2sev']  = dict(dist='lognormal_int', par1=6.6, par2=4.9) # Duration from symptomatic to severe symptoms; see Linton et al., https://doi.org/10.3390/jcm9020538
     pars['dur']['sev2crit'] = dict(dist='lognormal_int', par1=3.0, par2=7.4) # Duration from severe symptoms to requiring ICU; see Wang et al., https://jamanetwork.com/journals/jama/fullarticle/2761044
@@ -104,7 +104,7 @@ def make_pars(set_prognoses=False, prog_by_age=True, version=None, **kwargs):
 
     # If version is specified, load old parameters
     if version is not None:
-        version_pars = cvm.get_version_pars(version)
+        version_pars = cvm.get_version_pars(version, verbose=pars['verbose'])
         for key in pars.keys(): # Only loop over keys that have been populated
             if key in version_pars: # Only replace keys that exist in the old version
                 pars[key] = version_pars[key]
@@ -112,7 +112,7 @@ def make_pars(set_prognoses=False, prog_by_age=True, version=None, **kwargs):
     return pars
 
 
-# Define which parametrs need to be specified as a dictionary by layer -- define here so it's available at the module level for sim.py
+# Define which parameters need to be specified as a dictionary by layer -- define here so it's available at the module level for sim.py
 layer_pars = ['beta_layer', 'contacts', 'dynam_layer', 'iso_factor', 'quar_factor']
 
 
@@ -229,17 +229,16 @@ def get_prognoses(by_age=True, version=None):
 
     # If version is specified, load old parameters
     if version is not None:
-        version_pars = cvm.get_version_pars(version)
-        version_prognoses = version_pars['prognoses']
+        version_prognoses = cvm.get_version_pars(version, verbose=False)['prognoses']
         for key in version_prognoses.keys(): # Only loop over keys that have been populated
-            if key in version_pars: # Only replace keys that exist in the old version
-                prognoses[key] = version_pars[key]
+            if key in version_prognoses: # Only replace keys that exist in the old version
+                prognoses[key] = np.array(version_prognoses[key])
 
     # Check that lengths match
     expected_len = len(prognoses['age_cutoffs'])
     for key,val in prognoses.items():
         this_len = len(prognoses[key])
-        if this_len != expected_len:
+        if this_len != expected_len: # pragma: no cover
             errormsg = f'Lengths mismatch in prognoses: {expected_len} age bins specified, but key "{key}" has {this_len} entries'
             raise ValueError(errormsg)
 
